#!/usr/bin/perl -w

# LSB Test Execution Framework
# AJAX Server Module (ajax_srv.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my $error = "";
my $data = "";

my $output_xml = <<DATA;
HTTP/1.0 200 OK
Content-type: text/xml

<?xml version="1.0" encoding="UTF-8"?>
DATA

sub list_dir() {
	if (opendir(DIR, $SERVER_PARAM{"APP_DATA"}."/profiles")) {
		$data .= "<profiles>\n";
		foreach (sort(grep(!/^[~\.]/, readdir(DIR)))) {
			$data .= "<name>$_</name>\n";
		}
		$data .= "</profiles>\n";
		closedir(DIR);
	}
	else {
		$error = "Could not obtain directory list!<br />($!)";
	}
}

# Helps to determine platform name.
sub guess_platform($) {
    if ($_[0] =~ /^(amd64|x86_64)$/) {
		return "x86_64";
	}
    elsif ($_[0] =~ /^(ia32|i[3-6]86|x86)$/) {
		return "ia32";
	}
    elsif ($_[0] =~ /^(ia64)$/) {
		return "ia64";
	}
    elsif ($_[0] =~ /^(ppc|ppc32)$/) {
		return "ppc32";
	}
    elsif ($_[0] =~ /^(ppc64)$/) {
		return "ppc64";
	}
    elsif ($_[0] =~ /^(s390)$/) {
		return "s390";
	}
    elsif ($_[0] =~ /^(s390x)$/) {
		return "s390x";
	}
    else {
		return "unknown";
	}
}

if (!$_GET{"action"}) {
	$error = "Incorrect call parameters!";
}
elsif ($_GET{"action"} eq "delete") {
	if ($_GET{"profile"}) {
		if (!-f $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"}) {
			$error = "File not found:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		}
		else {
			unlink($SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"});
			list_dir();
		}
	}
	else {
		$error = "No profile name specified!";
	}
}
elsif ($_GET{"action"} eq "save") {
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
		if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
			$error = "Could not create profile directory:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles";
		}
	}
	if (!$error) {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/<>\|\?\*]/) {
			$error = "Invalid profile name: ".$_POST{"edit_profile_name"};
		}
	}
	if (!$error) {
		if ($_POST{"application"} and ($_POST{"application"} =~ m/[\\\/<>\|\?\*]/)) {
			$error = "Invalid application name: ".$_POST{"application"};
		}
	}
	if (!$error) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"};
		if (open(FILE, ">$full_save_name")) {
			my $architecture = ($_POST{"architecture"} or guess_platform(`uname -m`));
			my $lsb_version  = ($_POST{"lsb_version"} or "3.1");
			my $lsb_profile  = ($_POST{"lsb_profile"} or "desktop");
			print FILE "APPLICATION:     ".$_POST{"application"}."\n" if ($_POST{"application"});
			if ($_POST{"paths"}) {
				$_POST{"paths"} =~ s/\r\n/\n/g;
				my @paths_lines = split(/\n/, $_POST{"paths"});
				foreach (@paths_lines) {
					print FILE "PATH:            $_\n";
				}
			}
			if ($_POST{"packages"}) {
				$_POST{"packages"} =~ s/\r\n/\n/g;
				my @packages_lines = split(/\n/, $_POST{"packages"});
				foreach (@packages_lines) {
					print FILE "PACKAGE:         $_\n";
				}
			}
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}
			close(FILE);
			list_dir();
		}
		else {
			$error = "Cannot open file for writing ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{"action"} eq "load") {
	my %profile_data = (
		"application" => "",
		"paths" => "",
		"packages" => "",
		"architecture" => "",
		"lsb_version" => "3.1",
		"lsb_profile" => "desktop",
		"comments" => ""
	);
	if (!$_GET{"profile"}) {
		$error = "No profile name specified!";
	}
	if (!$error) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		if (open(FILE, $full_save_name)) {
			my @comments = ();
			my @paths = ();
			my @packages = ();
			while (<FILE>) {
				s/[\r\n]//g;
				if (/^APPLICATION:\s+(.*)$/) {
					$profile_data{"application"} = $1;
				}
				elsif (/^PATH:\s+(.*)$/) {
					push @paths, $1;
				}
				elsif (/^PACKAGE:\s+(.*)$/) {
					push @packages, $1;
				}
				elsif (/^LSB_VERSION:\s+(.*)$/) {
					$profile_data{"lsb_version"} = $1;
				}
				elsif (/^LSB_PROFILE:\s+(.*)$/) {
					$profile_data{"lsb_profile"} = $1;
				}
				elsif (/^ARCHITECTURE:\s+(.*)$/) {
					$profile_data{"architecture"} = $1;
				}
				elsif (/^COMMENTS: (.*)$/) {
					push @comments, $1;
				}
			}
			close(FILE);
			$profile_data{"paths"}    = join("\n", @paths);
			$profile_data{"packages"} = join("\n", @packages);
			$profile_data{"comments"} = join("\n", @comments);
			$data .= "<profile_data>\n";
			foreach (keys %profile_data) {
				$data .= "<item><name>$_</name><value>".$profile_data{$_}."</value></item>\n";
			}
			$data .= "</profile_data>\n";
			list_dir();
		}
		else {
			$error = "Cannot open file for reading ($!):<br />$full_save_name";
		}
	}
}
else {
	$error = "Incorrect call parameters!";
}

if ($error) {
	$error = "<ul><li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$error</font></li></ul>";
	$error =~ s/&/&amp;/g;
	$error =~ s/</&lt;/g;
	$error =~ s/>/&gt;/g;
	print $output_xml."<root>\n<error>".$error."</error>\n</root>";
}
else {
	print $output_xml."<root>\n".$data."</root>";
}
